<?php
/*
 * @package		UNA API
 * @copyright	Copyright (C) 2023 Emir Sakic, http://www.sakic.net. All rights reserved.
 * @license		GNU/GPL, see LICENSE.TXT
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This header must not be removed. Additional contributions/changes
 * may be added to this header as long as no information is deleted.
 */

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

defined('_JEXEC') or die;

/**
 * Frontpage View class
 *
 * @since  1.5
 */
class UnaApiViewFrontpage extends JViewLegacy
{
	protected $state = null;

	protected $item = null;

	protected $items = null;

	protected $pagination = null;

	protected $lead_items = array();

	protected $intro_items = array();

	protected $link_items = array();

	/** @deprecated  4.0 */
	protected $columns = 1;

	/**
	 * An instance of JDatabaseDriver.
	 *
	 * @var    JDatabaseDriver
	 * @since  3.6.3
	 */
	protected $db;

	/**
	 * Execute and display a template script.
	 *
	 * @param   string  $tpl  The name of the template file to parse; automatically searches through the template paths.
	 *
	 * @return  mixed  A string if successful, otherwise an Error object.
	 */
	public function display($tpl = null)
	{
		$app = JFactory::getApplication('site');

		$state      = $this->get('State');
		$items      = $this->get('Items');
		$pagination = $this->get('Pagination');

		// Flag indicates to not add limitstart=0 to URL
		$pagination->hideEmptyLimitstart = true;

		// Check for errors.
		if (count($errors = $this->get('Errors')))
		{
			JError::raiseWarning(500, implode("\n", $errors));

			return false;
		}

		$params = &$state->params;

		// PREPARE THE DATA

		// Get the metrics for the structural page layout.
		$numLeading = (int) $params->def('num_leading_articles', 1);
		$numIntro   = (int) $params->def('num_intro_articles', 4);

		JPluginHelper::importPlugin('content');

		// Compute the article slugs and prepare introtext (runs content plugins).
		foreach ($items as &$item)
		{
			$item->slug        = $item->alias ? ($item->id . ':' . $item->alias) : $item->id;
			$item->catslug     = $item->category_alias ? ($item->catid . ':' . $item->category_alias) : $item->catid;
			$item->parent_slug = $item->parent_alias ? ($item->parent_id . ':' . $item->parent_alias) : $item->parent_id;

			// No link for ROOT category
			if ($item->parent_alias === 'root')
			{
				$item->parent_slug = null;
			}

			$item->event = new stdClass;

			// Old plugins: Ensure that text property is available
			if (!isset($item->text))
			{
				$item->text = $item->introtext;
			}

			if (!class_exists('JEventDispatcher')) {
				// Joomla 4
				Factory::getApplication()->triggerEvent('onContentPrepare', array('com_content.featured', &$item, &$item->params, 0));

				// Old plugins: Use processed text as introtext
				$item->introtext = $item->text;

				$results = Factory::getApplication()->triggerEvent('onContentAfterTitle', array('com_content.featured', &$item, &$item->params, 0));
				$item->event->afterDisplayTitle = trim(implode("\n", $results));

				$results = Factory::getApplication()->triggerEvent('onContentBeforeDisplay', array('com_content.featured', &$item, &$item->params, 0));
				$item->event->beforeDisplayContent = trim(implode("\n", $results));

				$results = Factory::getApplication()->triggerEvent('onContentAfterDisplay', array('com_content.featured', &$item, &$item->params, 0));
				$item->event->afterDisplayContent = trim(implode("\n", $results));
			} else {
				$dispatcher  = JEventDispatcher::getInstance();

				$dispatcher->trigger('onContentPrepare', array ('com_content.featured', &$item, &$item->params, 0));

				// Old plugins: Use processed text as introtext
				$item->introtext = $item->text;

				$results = $dispatcher->trigger('onContentAfterTitle', array('com_content.featured', &$item, &$item->params, 0));
				$item->event->afterDisplayTitle = trim(implode("\n", $results));

				$results = $dispatcher->trigger('onContentBeforeDisplay', array('com_content.featured', &$item, &$item->params, 0));
				$item->event->beforeDisplayContent = trim(implode("\n", $results));

				$results = $dispatcher->trigger('onContentAfterDisplay', array('com_content.featured', &$item, &$item->params, 0));
				$item->event->afterDisplayContent = trim(implode("\n", $results));
			}
		}

		// Preprocess the breakdown of leading, intro and linked articles.
		// This makes it much easier for the designer to just interogate the arrays.
		$max = count($items);

		// The first group is the leading articles.
		$limit = $numLeading;

		for ($i = 0; $i < $limit && $i < $max; $i++)
		{
			$this->lead_items[$i] = &$items[$i];
		}

		// The second group is the intro articles.
		$limit = $numLeading + $numIntro;

		// Order articles across, then down (or single column mode)
		for ($i = $numLeading; $i < $limit && $i < $max; $i++)
		{
			$this->intro_items[$i] = &$items[$i];
		}

		$this->columns = max(1, $params->def('num_columns', 1));
		$order = $params->def('multi_column_order', 1);

		if ($order == 0 && $this->columns > 1)
		{
			// Call order down helper
			$this->intro_items = UnaQueryHelper::orderDownColumns($this->intro_items, $this->columns);
		}

		// The remainder are the links.
		for ($i = $numLeading + $numIntro; $i < $max; $i++)
		{
			$this->link_items[$i] = &$items[$i];
		}

		$this->items      = &$items;
		$this->pagination = &$pagination;

		unset($item);
		$top_items = array();
		foreach ($items as $item) {

			$thumb = $image = '';
			$images = json_decode($item->images);
			if (isset($images->image_intro) && $images->image_intro) {
				$image = $images->image_intro;
			} else if (isset($images->image_fulltext) && $images->image_fulltext) {
				$image = $images->image_fulltext;
			}

			if ($image) {
				$thumb = JUri::base() . UnaApiHelper::imgSafe($image);
			}

			$top_items[] = array(
				'id' => $item->id,
				'category' => $item->category_title,
				'categoryId' => $item->catid,
				'title' => $item->title,
				'url' => JRoute::_(ContentHelperRoute::getArticleRoute($item->slug, $item->catid, $item->language), true, JRoute::TLS_IGNORE, true),
				'datePublished' => HtmlHelper::date($item->publish_up, 'Y-m-d H:i:s', null),
				'dominantColor' => '',
				'thumb' => $thumb,
				'gallery' => array(
					'photo' => 0,
					'video' => 0
				),
				'comments' => array(
					'show' => 0,
					'count' => 0
				),
			);
		}

		// get categories
		JLoader::register( 'ContentModelCategories', JPATH_SITE . '/components/com_content/models/categories.php' );
		if (!class_exists('ContentModelCategories')) {
			// Joomla 4
			$categories_model = JFactory::getApplication()->bootComponent('com_content')->getMVCFactory()->createModel('Categories', 'Site');
		} else {
			// Joomla 3
			$categories_model = JModelLegacy::getInstance('ContentModelCategories');
		}

		$categories = $categories_model->getItems(true);

		$app->input->set('limit', 3);
		$articles_model = JModelLegacy::getInstance('Articles', 'UnaApiModel');
		$articles_model->setState('list.limit', 3);

		$includeCategories = $params->get('include_categories', array());

		$latest_items = array();
		foreach ($categories as $category) {

			if (in_array($category->id, $includeCategories)) {

				$articles_model->setState('filter.category_id', $category->id);
				$articles_model->setState('filter.subcategories', true);  // include subcategories

				$items = $articles_model->getItems();
				foreach ($items as $item) {

					// Compute the article slugs and prepare introtext (runs content plugins).
					$item->slug        = $item->alias ? ($item->id . ':' . $item->alias) : $item->id;
					$item->catslug     = $item->category_alias ? ($item->catid . ':' . $item->category_alias) : $item->catid;
					$item->parent_slug = $item->parent_alias ? ($item->parent_id . ':' . $item->parent_alias) : $item->parent_id;

					// No link for ROOT category
					if ($item->parent_alias === 'root')
					{
						$item->parent_slug = null;
					}

					// Old plugins: Ensure that text property is available
					if (!isset($item->text))
					{
						$item->text = $item->introtext;
					}

					$thumb = $image = '';
					$images = json_decode($item->images);
					if (isset($images->image_intro) && $images->image_intro) {
						$image = $images->image_intro;
					} else if (isset($images->image_fulltext) && $images->image_fulltext) {
						$image = $images->image_fulltext;
					}

					if ($image) {
						$thumb = JUri::base() . UnaApiHelper::imgSafe($image);
					}
					$latest_items[] = array(
						'id' => $item->id,
						'category' => $category->title,
						'categoryId' => $category->id,
						'title' => $item->title,
						'url' =>  JRoute::_(ContentHelperRoute::getArticleRoute($item->slug, $item->catid, $item->language), true, JRoute::TLS_IGNORE, true),
						'datePublished' => HtmlHelper::date($item->publish_up, 'Y-m-d H:i:s', null),
						'dominantColor' => '',
						'thumb' => $thumb,
						'gallery' => array(
							'photo' => 0,
							'video' => 0
						),
						'comments' => array(
							'show' => 0,
							'count' => 0
						),
					);
				}

			}
		}

		echo json_encode(array(
			'newsTop' => $top_items,
			'newsNewest' => $latest_items
		));
	}
}